'use strict';

const _ = require('lodash');
const Boom = require('boom');
const debug = require('debug')('hapi-semver-api:routes');

function routesAddVersions(routesList) {

  debug('start transform for routes array with length =', routesList.length);

  return routesList.reduce((acc, currPathDirty) => {

    const since = (currPathDirty.version && currPathDirty.version.since) ? currPathDirty.version.since : '1.0.0';

    const currPath = _.omit(_.cloneDeep(currPathDirty), ['version']);

    const existedRoute = acc.find((x) => (x.path === currPath.path && x.method === currPath.method));

    if (existedRoute) { // just add new version
      if (existedRoute.config.plugins.version.list.indexOf(since) !== -1) {
        throw new Error(`Hapi Router Version: version "${since}" already exists for route "${currPath.path}"`);
      }
      debug('Route "%s %s" already exists, added version "%s"', currPath.method, currPath.path, since);
      existedRoute.config.plugins.version.list.push(since);
      if (!currPath.config) {
        currPath.config = {};
      }
      if (!currPath.config.tags) {
        currPath.config.tags = [];
      }
      currPath.path = `/v${since}${currPath.path}`;
      currPath.config.tags.push('swagger-skip-route');

      acc.push(currPath);
      return acc;
    }

    const plainPath = _.cloneDeep(currPath);

    plainPath.handler = async function unusedHandler(request, reply) {
      return reply(Boom.badRequest(`Path '${request.path}' already versioned and must not be reached without version header`));
    };

    if (!plainPath.config) {
      plainPath.config = {};
    }
    if (!plainPath.config.tags) {
      plainPath.config.tags = [];
    } else {
      plainPath.config.tags = Object.assign([], currPath.config.tags);
    }

    if (!plainPath.config.plugins) {
      plainPath.config.plugins = {};
    }
    if (!plainPath.config.plugins.version) {
      plainPath.config.plugins.version = {};
    }
    if (!plainPath.config.plugins.version.list) {
      plainPath.config.plugins.version.list = [];
    }

    debug('Added "%s %s" with version "%s"', currPath.method, currPath.path, since);

    plainPath.config.plugins.version.list.push(since);

    if (!currPath.config) {
      currPath.config = {};
    }
    if (!currPath.config.tags) {
      currPath.config.tags = [];
    }
    currPath.path = `/v${since}${currPath.path}`;
    currPath.config.tags.push('swagger-skip-route');

    acc.push(plainPath);
    acc.push(currPath);

    return acc;
  }, []);
}

module.exports = { routesAddVersions };
